
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

/**
 * 
 */
package com.eu.ales.desyre.builder.ipprovider.meta;


import java.io.File;
import java.io.IOException;
import java.net.URI;

import org.apache.log4j.Logger;

import com.eu.ales.desyre.common.DesyreException;
import com.eu.ales.desyre.common.IImplementationProvider;
import com.eu.ales.desyre.configuration.DesyreConfiguration;
import com.eu.ales.desyre.utils.FileUtils;
import com.eu.ales.desyre.utils.StringHelper;


/**
 * @author sofronis
 *
 */
public class METAFileBasedImplementationProvider implements IImplementationProvider {
	Logger logger = Logger.getLogger(METAFileBasedImplementationProvider.class);

	private File localWorkspace;
	private File bundlesDir;

	/**
	 * @param tmpDir
	 */
	public METAFileBasedImplementationProvider(File localWorkspace, File bundlesDir) {
		this.localWorkspace = localWorkspace;
		this.bundlesDir = bundlesDir;
		
		if ( !bundlesDir.exists() )
			logger.warn("Bundles directory does not exist: " + bundlesDir);
	}

	/* (non-Javadoc)
	 * @see com.eu.ales.desyre.common.IImplementationProvider#getImplementation(java.net.URI)
	 */
	@Override
	public File getImplementation(URI implId) throws DesyreException {
		
		String path = implId.getPath();
		// correct the path if a namespace suffix has been added
		if ( DesyreConfiguration.APPEND_NAMESPACES_ON_COMPONENTS_NAME && path.endsWith(DesyreConfiguration.COMPONENT_NAMESPACE_SUFFIX))
			path = StringHelper.deleteTrailing(path, DesyreConfiguration.COMPONENT_NAMESPACE_SUFFIX.length());
		path = path.substring(1).replace("/", ".") + "." + DesyreConfiguration.IMPLEMENTATION_BUNDLE_EXTENSION;
		
		File srcSarFile = new File( bundlesDir, path );
		if ( srcSarFile.exists() ) {
			File trgSarFile = new File( localWorkspace, srcSarFile.getName() );
			try {
				FileUtils.copyFile(srcSarFile, trgSarFile);
				
			} catch (IOException e) {
				throw new DesyreException(e);
			}
			
			return trgSarFile;
			
		} else {
			throw new DesyreException("Could not find Sar file for " + implId);
		}
		
	}
}
