
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.desyregenerator;

import com.eu.ales.rpyplugins.desyregenerator.impl.DesyreGenerator;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPClass;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.RPExtendedRPClassesFactory;
import com.telelogic.rhapsody.core.RPUserPlugin;

public class DesyreGeneratorPlugIn extends RPUserPlugin {
	private IRPApplication rhpApplication;
	private Logger logger;

	@Override
	public void RhpPluginInit(IRPApplication rpyApplication) {
		this.rhpApplication = rpyApplication;
		this.logger = Logger.getLogger(this.rhpApplication, DesyreGeneratorPlugIn.class);
		logger.logInfo("Initialization");
	}

	@Override
	public void RhpPluginInvokeItem() {
		logger.logInfo("Invoke Item");
		DesyreGenerator desyreGenerator = new DesyreGenerator(rhpApplication);
		com.eu.ales.desyre.model.desyre.System desyreSystem = desyreGenerator.loadDesyreModel();
		desyreGenerator.generate(desyreSystem);
		//desyreGenerator.dessbuild();
		logger.logInfo("Invoke Item End");
	}

	@Override
	public void OnMenuItemSelect(String menuItem) {
		// TODO Auto-generated method stub
		logger.logInfo("On Menu Item Select [" + menuItem + "]");
		synchronized (RPExtendedRPClassesFactory.factoryMutex) {
			AssertionUtils.assumption(this.rhpApplication != null, "Invalid operation on null Rhapsody application");
			final IRPModelElement selectedIRPModelElement = this.rhpApplication.getSelectedElement();
			if (selectedIRPModelElement instanceof IRPClass) {
				IRPClass irpClass = (IRPClass) selectedIRPModelElement;
				logger.logWarning("On Menu Item Select ['" + irpClass.getName() + "'] is not supported");
			} else {
				logger.logWarning("selection of model elements of type '" + selectedIRPModelElement.getClass().getSimpleName() + "' is not supported");
			}
		}
		logger.logInfo("On Menu Item Select [" + menuItem + "] End");
	}

	@Override
	public void OnTrigger(String trigger) {
		// TODO Auto-generated method stub
		logger.logInfo("On Trigger [" + trigger + "]");
		logger.logInfo("On Trigger [" + trigger + "] End");
	}

	@Override
	public boolean RhpPluginCleanup() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void RhpPluginFinalCleanup() {
		// TODO Auto-generated method stub
	}

}
