
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.desyregenerator.impl;

import java.io.File;
import java.io.IOException;

import org.eclipse.emf.ecore.EObject;

import com.eu.ales.desyre.builder.DesyreBuilder;
import com.eu.ales.desyre.builder.DesyreBuilderException;
import com.eu.ales.desyre.configuration.DesyrePathHelper;
import com.eu.ales.desyre.model.desyre.DesyrePackage;
import com.eu.ales.rpyplugins.utils.FileManager;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.rpyplugins.utils.Serialization;
import com.eu.ales.utils.AssertionUtils;
import com.telelogic.rhapsody.core.IRPApplication;

public class DesyreGenerator {
	private Logger logger;
	private DesyreBuilder builder;

	public DesyreGenerator(IRPApplication irpApplication) {
		this.logger = Logger.getLogger(irpApplication, DesyreGenerator.class);
		this.builder = null;
	}

	public com.eu.ales.desyre.model.desyre.System loadDesyreModel() {
		File xmiFile = FileManager.createDesyreWorkspaceFile(FileManager.DESYRE_RESOURCE_FILE_NAME + FileManager.RESOURCE_FILE_EXTENSION);
		com.eu.ales.desyre.model.desyre.System desyreSystem = null;
		EObject model = null;
		try {
			model = Serialization.loadRootFromFile(xmiFile, DesyrePackage.eNS_URI, DesyrePackage.eINSTANCE);
			desyreSystem = (com.eu.ales.desyre.model.desyre.System)model;
		} catch (IOException e) {
			logger.logError("IOException while loading model from file '" + xmiFile.getAbsolutePath() + "': " + e.getMessage());
		} catch (ClassCastException e) {
			logger.logError("Invalid casting '" + model.eClass().getName() + "' to 'com.eu.ales.desyre.model.desyre.System' while loading model from file '" + xmiFile.getAbsolutePath() + "': " + e.getMessage());
		}
		return desyreSystem;
	}

	public void generate(com.eu.ales.desyre.model.desyre.System desyreSystem) {
		AssertionUtils.assumption(desyreSystem != null, "Invalid null DESYRE ECore model");
		DesyrePathHelper desyrePathHelper;
		desyrePathHelper = new DesyrePathHelper(FileManager.getDesyreSimulatorDir());
		desyrePathHelper.setImplementationRepositoryPath(FileManager.getDesyreImplementationRepository().getAbsolutePath());
		this.logger.logTrace("DESYRE      Simulator Directory='" + FileManager.getDesyreSimulatorDir() + "'");
		this.logger.logTrace("DESYRE Implementation Directory='" + FileManager.getDesyreImplementationRepository().getAbsolutePath() + "'");
		DesyreBuilder builder = new DesyreBuilder(desyrePathHelper);
		try {
			// construct DESYRE 
			builder.constructDesyre(desyreSystem);
		} catch (DesyreBuilderException e) {
			this.logger.logInfo("DesyreBuilderException while constructing DESYRE model: " + e.getMessage());
		}
	}

	public void dessbuild() {
		AssertionUtils.assumption(this.builder != null, "Invalid operation: call function 'generate(...)' first");
		// build DESYRE simulator
		this.builder.desyreIIBuild();
	}
}
