
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.flows.metaii2desyre;

import com.eu.ales.rpyplugins.rpyparser.RpyParserPlugIn;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPClass;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.RPExtendedRPClassesFactory;
import com.telelogic.rhapsody.core.RPUserPlugin;

public class MetaII2DesyrePlugIn extends RPUserPlugin {
	private IRPApplication rpyApplication;
	private Logger logger;

	@Override
	public void RhpPluginInit(IRPApplication rpyApplication) {
		this.rpyApplication = rpyApplication;
		this.logger = Logger.getLogger(this.rpyApplication, MetaII2DesyrePlugIn.class);
		logger.logInfo("Initialization");
	}

	@Override
	public void RhpPluginInvokeItem() {
		logger.logInfo("Invoke Item");

		com.eu.ales.rpyplugins.metaiivalidator.MetaIIValidatorPlugIn metaIIValidatorPlugIn = new com.eu.ales.rpyplugins.metaiivalidator.MetaIIValidatorPlugIn();
		com.eu.ales.rpyplugins.metaii2desyre.MetaII2DesyrePlugIn metaII2DesyrePlugIn = new com.eu.ales.rpyplugins.metaii2desyre.MetaII2DesyrePlugIn();
		com.eu.ales.rpyplugins.desyre2modelica.Desyre2ModelicaPlugIn desyre2ModelicaPlugIn = new com.eu.ales.rpyplugins.desyre2modelica.Desyre2ModelicaPlugIn();
		com.eu.ales.rpyplugins.modelicagenerator.ModelicaGeneratorPlugIn modelicaGeneratorPlugIn = new com.eu.ales.rpyplugins.modelicagenerator.ModelicaGeneratorPlugIn();
		com.eu.ales.rpyplugins.desyregenerator.DesyreGeneratorPlugIn desyreGeneratorPlugIn = new com.eu.ales.rpyplugins.desyregenerator.DesyreGeneratorPlugIn();

		if (rpyApplication != null) {
			// Parsing Rhapsody model into METAII model
			RpyParserPlugIn rpyParserPlugIn = new RpyParserPlugIn();
			rpyParserPlugIn.RhpPluginInit(rpyApplication);
			rpyParserPlugIn.RhpPluginInvokeItem();
		}

		// Validating METAII model
		metaIIValidatorPlugIn.RhpPluginInit(rpyApplication);
		metaIIValidatorPlugIn.RhpPluginInvokeItem();

		// Translating METAII model into DESYRE model
		metaII2DesyrePlugIn.RhpPluginInit(rpyApplication);
		metaII2DesyrePlugIn.RhpPluginInvokeItem();

		// Translating DESYRE model to Modelica model
		desyre2ModelicaPlugIn.RhpPluginInit(rpyApplication);
		desyre2ModelicaPlugIn.RhpPluginInvokeItem();

		// Generating Modelica code
		modelicaGeneratorPlugIn.RhpPluginInit(rpyApplication);
		modelicaGeneratorPlugIn.RhpPluginInvokeItem();

		// Generating DESYRE code
		desyreGeneratorPlugIn.RhpPluginInit(rpyApplication);
		desyreGeneratorPlugIn.RhpPluginInvokeItem();
		
		logger.logInfo("Invoke Item End");
	}

	@Override
	public void OnMenuItemSelect(String menuItem) {
		// TODO Auto-generated method stub
		logger.logInfo("On Menu Item Select [" + menuItem + "]");
		synchronized (RPExtendedRPClassesFactory.factoryMutex) {
			AssertionUtils.assumption(this.rpyApplication != null, "Invalid operation on null Rhapsody application");
			final IRPModelElement selectedIRPModelElement = this.rpyApplication.getSelectedElement();
			if (selectedIRPModelElement instanceof IRPClass) {
				IRPClass irpClass = (IRPClass) selectedIRPModelElement;
				logger.logWarning("On Menu Item Select ['" + irpClass.getName() + "'] is not supported");
			} else {
				logger.logWarning("selection of model elements of type '" + selectedIRPModelElement.getClass().getSimpleName() + "' is not supported");
			}
		}
		logger.logInfo("On Menu Item Select [" + menuItem + "] End");
	}

	@Override
	public void OnTrigger(String trigger) {
		// TODO Auto-generated method stub
		logger.logInfo("On Trigger [" + trigger + "]");
		logger.logInfo("On Trigger [" + trigger + "] End");
	}

	@Override
	public boolean RhpPluginCleanup() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void RhpPluginFinalCleanup() {
		// TODO Auto-generated method stub
	}

}
