
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import com.eu.ales.desyre.model.desyre.ConfigurationElement;
import com.eu.ales.desyre.model.desyre.DesyreFactory;
import com.eu.ales.desyre.model.desyre.Litteral;
import com.eu.ales.modeltransformation.impl.MappingImpl;
import com.eu.ales.rpyplugins.MAttribute;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;
import com.eu.ales.utils.AssumptionViolation;

public class Attribute2ConfigurationElement extends MappingImpl<MAttribute, ConfigurationElement> {
	private static final Logger LOGGER = Logger.getLogger(Attribute2ConfigurationElement.class);
	public static final String MULTIPLICITY_MANY = "*";

	public Attribute2ConfigurationElement(MAttribute self) {
		super(self);
	}

	@Override
	public void populate() {
		setConfigurationElement(self, result);
	}

	public static ConfigurationElement createConfigurationElement(MAttribute attribute) {
		return setConfigurationElement(attribute, DesyreFactory.eINSTANCE.createConfigurationElement());
	}

	public static ConfigurationElement setConfigurationElement(MAttribute attribute, ConfigurationElement configurationElement) {
		AssertionUtils.assumption(attribute != null, "Invalid null MetaII attribute argument");
		AssertionUtils.assumption(configurationElement != null, "Invalid null DESYRE configuration element argument");
		Litteral value = DesyreFactory.eINSTANCE.createLitteral();
		value.setValue(attribute.getName());
		configurationElement.setReferenceId(value);
		String attributeValue = Attribute2Parameter.getAttributeValue(attribute);
		if (attributeValue != null) {
			value = DesyreFactory.eINSTANCE.createLitteral();
			value.setValue(attributeValue);
			configurationElement.setConfigurableElementValue(value);
		}
		return configurationElement;
	}
}
