
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import java.util.List;

import com.eu.ales.desyre.model.desyre.DataType;
import com.eu.ales.desyre.model.desyre.DeclarationZone;
import com.eu.ales.desyre.model.desyre.DesyreFactory;
import com.eu.ales.desyre.model.desyre.Litteral;
import com.eu.ales.desyre.model.desyre.Parameter;
import com.eu.ales.desyre.model.desyre.PrimitiveType;
import com.eu.ales.modeltransformation.impl.DerivedMappingImpl;
import com.eu.ales.rpyplugins.MAttribute;
import com.eu.ales.rpyplugins.MLiteralSpecification;
import com.eu.ales.rpyplugins.MType;
import com.eu.ales.rpyplugins.MValueSpecification;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;

public class Attribute2Parameter extends DerivedMappingImpl<MAttribute, Parameter> {
	private static final Logger LOGGER = Logger.getLogger(Attribute2Parameter.class);
	public static final String MULTIPLICITY_MANY = "*";

	private DeclarationZone desyreDeclarationZone;

	public Attribute2Parameter(MAttribute self) {
		super(self);
		this.desyreDeclarationZone = null;
	}

	public Attribute2Parameter(MAttribute self, DeclarationZone declarationZone) {
		this(self);
		this.desyreDeclarationZone = declarationZone;
	}

	@Override
	public void populate() {
		result.setName(self.getName());
		DataType dataType = new Classifier2DataType(self.getType()).mapIfNotExists(desyreDeclarationZone);
		result.setType(dataType);
		setParameter(self, result, false /*do not check types*/);
	}

	@Override
	protected void initInheritance() {
		inherits(ModelElement2NamedElement.class);
	}

	public static void setParameter(MAttribute attribute, Parameter parameter, boolean checkType) {
		AssertionUtils.assumption(attribute != null, "Invalid null attribute argument");
		AssertionUtils.assumption(parameter != null, "Invalid null parameter argument");
		AssertionUtils.assumption(attribute.getName().equals(parameter.getName()),
				"MetaII attribute name '" + attribute.getName() + "' does not match DESYRE parameter name '" + parameter.getName() + "'");
		if (checkType) {
			if (attribute.getType() instanceof MType && Type2PrimitiveType.when((MType)attribute.getType())) {
				if (parameter.getType() instanceof PrimitiveType) {
					if (Type2PrimitiveType.getPrimitiveTypeKind((MType)attribute.getType()) != ((PrimitiveType)parameter.getType()).getKind()) {
						LOGGER.logError(attribute, "type mismatch: attribute type='" + attribute.getType().getName() + "', parameter type='" + parameter.getType().getName() + "'");
					}
				} else if (parameter.getType() == null) {
					LOGGER.logError(attribute, "type mismatch: attribute type='" + attribute.getType().getName() + "', parameter type='null': forcing parameter's type to attribute's type");
					DataType type = new Classifier2DataType(attribute.getType()).mapIfNotExists();
					parameter.setType(type);
				} else {
					LOGGER.logError(attribute, "DESYRE parameter type '" + attribute.getType().getName() + "' is non-primitive");
				}
			} else {
				LOGGER.logError(attribute, "unsupported non-primitive type '" + attribute.getType().getName() + "'");
			}
		}
		String attributeValue = getAttributeValue(attribute);
		if (attributeValue != null) {
			Litteral value = DesyreFactory.eINSTANCE.createLitteral();
			value.setValue(attributeValue);
			parameter.setValue(value);
		}
	}

	public static String getAttributeValue(MAttribute attribute) {
		List<MValueSpecification> valueSpecifications = attribute.getMValueSpecifications();
		if (valueSpecifications.size() == 1) {
			MValueSpecification valueSpecification = valueSpecifications.get(0);
			if (valueSpecification instanceof MLiteralSpecification) {
				return ((MLiteralSpecification)valueSpecification).getValue();
			} else if (valueSpecification == null) {
				LOGGER.logError(attribute, "unexpecte null value specification");
			} else {
				LOGGER.logError(attribute, "unsupported value specification of type '" + valueSpecification.eClass().getName() + "'");
			}
		} else if (valueSpecifications.size() == 0) {
			LOGGER.logError(attribute, "parameter has no value specification");
		} else if (valueSpecifications.size() > 1) {
			LOGGER.logError(attribute, "parameters with multiple value specifications are not supported");
		}
		return null;
	}
}
