
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.eu.ales.desyre.model.desyre.ArrayType;
import com.eu.ales.desyre.model.desyre.Component;
import com.eu.ales.desyre.model.desyre.ComponentProperty;
import com.eu.ales.desyre.model.desyre.ConfigurationElement;
import com.eu.ales.desyre.model.desyre.DataType;
import com.eu.ales.desyre.model.desyre.DeclarationZone;
import com.eu.ales.desyre.model.desyre.DesyreFactory;
import com.eu.ales.desyre.model.desyre.Link;
import com.eu.ales.desyre.model.desyre.Litteral;
import com.eu.ales.desyre.model.desyre.Parameter;
import com.eu.ales.desyre.model.desyre.Port;
import com.eu.ales.desyre.model.desyre.PortDirection;
import com.eu.ales.desyre.model.desyre.PortKind;
import com.eu.ales.desyre.model.desyre.PrimitiveType;
import com.eu.ales.desyre.model.desyre.PrimitiveTypeKind;
import com.eu.ales.desyre.model.desyre.ReusableElement;
import com.eu.ales.desyre.model.desyre.System;
import com.eu.ales.desyre.model.desyre.View;
import com.eu.ales.desyre.model.desyre.util.StructureHelper;
import com.eu.ales.modeltransformation.impl.DerivedMappingImpl;
import com.eu.ales.rpyplugins.MAttribute;
import com.eu.ales.rpyplugins.MClass;
import com.eu.ales.rpyplugins.MClassifier;
import com.eu.ales.rpyplugins.MInstance;
import com.eu.ales.rpyplugins.MLink;
import com.eu.ales.rpyplugins.MModelElement;
import com.eu.ales.rpyplugins.MPort;
import com.eu.ales.rpyplugins.MProject;
import com.eu.ales.rpyplugins.MStereotype;
import com.eu.ales.rpyplugins.MSysMLPort;
import com.eu.ales.rpyplugins.MTag;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.rpyplugins.utils.ModelUtils;
import com.eu.ales.utils.AssertionUtils;

public class Classifier2Component extends DerivedMappingImpl<MClassifier, Component> {
	private static final Logger LOGGER = Logger.getLogger(Classifier2Component.class);;
	public static final String MOCC_STEREOTYPE_NAME = "MoCC";
	public static final String ADAPTOR_STEREOTYPE_NAME = "Adaptor";
	public static final String MULTICAST_STEREOTYPE_NAME = "Multicast";
	public static final String REPLICATOR_STEREOTYPE_NAME = "Replicator";
	public static final String TEMPLATE_PORT_TYPE = "template_port_type";
	public static final String FMU_FILE_PATH = "fmu_file_path";
	public static final String MODELICA_FMU_PATH = "METAII" + File.separator + "ModelicaFmu";
	public static final String MODELICA_FMU_EXTENSION = ".fmu";
	private static final Map<MModelElement, Set<MStereotype>> componentToAllStereotypesAndTheirGeneralizationsMap = new HashMap<MModelElement, Set<MStereotype>>();;
	public static MStereotype moccStereotype = null;
	public static MStereotype adaptorStereotype = null;
	public static MStereotype multicastStereotype = null;
	public static MStereotype replicatorStereotype = null;
	private String desyreDeclarationZonePath;

	public Classifier2Component(MClassifier self) {
		super(self);
		this.desyreDeclarationZonePath = null;
	}

	public Classifier2Component(MClassifier self, String desyreDeclarationZonePath) {
		this(self);
		this.desyreDeclarationZonePath = desyreDeclarationZonePath;
	}
static int counter = 0;
	@Override
	public void populate() {
		AssertionUtils.assumption(desyreDeclarationZonePath != null, "Invalid null DESYRE library name");
		AssertionUtils.assumption(self instanceof MClass, "Unsupported Classifier '" + self.eClass().getName() + "': MClass expected");

		// first get DESYRE library object by name
		DeclarationZone desyreDeclarationZone = MetaII2DesyreTransformation.getDesyreDeclarationZone(desyreDeclarationZonePath);
		result.setName(NameMapping.getName(self));
		// add component to library
		desyreDeclarationZone.getDeclared().add(result);
		int numberOfInputPorts = 0;
		int numberOfOtherPorts = 0;
		for (MModelElement modelElement : self.getMNestedElements()) {
			if (modelElement instanceof MSysMLPort) {
				if (((MSysMLPort)modelElement).getPortDirection().equals(SysMLPort2Port.IN)) {
					numberOfInputPorts++;
				} else {
					numberOfOtherPorts++;
				}
			}
		}
		for (MModelElement modelElement : self.getMNestedElements()) {
			if (modelElement instanceof MSysMLPort) {
				// add all ports to the component
				Port port = new Classifier2Port(self, (MSysMLPort)modelElement, result.getDeclarationZone()).mapIfNotExists();
				if (port == null) {
					port = new SysMLPort2Port((MSysMLPort)modelElement, self, result.getDeclarationZone()).map();
				} else if (Tag2Target.isSimulinkTarget(self) && port.getType() instanceof ArrayType) {
					if (port.getDirection() == PortDirection.IN) {
						((ArrayType)port.getType()).setSize(numberOfInputPorts);
					} else {
						((ArrayType)port.getType()).setSize(numberOfOtherPorts);
					}
				}
				result.getPort().add(port);
			}
		}
		View view = new Classifier2View(self).map();
		result.getView().add(view);
		result.setUri(view.getName() + "://" + desyreDeclarationZonePath + "/" + result.getName());
		// PROCESS LINKS
		for (MLink mLink : self.getMLinks()) {
			Type2PrimitiveType.resolveInheritedDataType(mLink);
			if (mLink.getFrom() != null && mLink.getTo() != null) {
				ComponentProperty from = getComponentProperty(mLink.getFrom());
				ComponentProperty to = getComponentProperty(mLink.getTo());
				int p = 0;
				if (from != null && from.getName().equals("GenL") || to != null && to.getName().equals("GenL")) {
					p++;
				}
				Port fromPort = null;
				MSysMLPort fromSysMLPort = mLink.getFromSysMLPort();
				if (fromSysMLPort == null && mLink.getFrom() instanceof MSysMLPort) {
					// this link is a hierarchical connection
					fromSysMLPort = (MSysMLPort)mLink.getFrom();
					fromPort = getPort(fromSysMLPort, result);
				} else {
					if (from != null) {
						fromPort = Classifier2Component.processPortForReplicator(fromSysMLPort, from.getType());
					}
					if (fromPort == null) {
						fromPort = Classifier2Component.processPortForMulticast(fromSysMLPort, from.getType());
					}
					if (fromPort == null) {
						fromPort = Classifier2Component.processPortForAdaptor(fromSysMLPort, from.getType());
					}
					if (fromPort == null) {
						fromPort = getPort(fromSysMLPort, from.getType());
					}
				}
				if (fromPort == null) {
					fromPort = new Classifier2Port(fromSysMLPort.getOfClass(), fromSysMLPort, null).getPort();
				}
				Port toPort = null;
				MSysMLPort toSysMLPort = mLink.getToSysMLPort();
				if (toSysMLPort == null && mLink.getTo() instanceof MSysMLPort) {
					// this link is a hierarchical connection
					toSysMLPort = (MSysMLPort)mLink.getTo();
					toPort = getPort(toSysMLPort, result);
				} else {
					if (to != null) {
						toPort = Classifier2Component.processPortForReplicator(toSysMLPort, to.getType());
					}
					if (toPort == null) {
						toPort = Classifier2Component.processPortForMulticast(toSysMLPort, to.getType());
					}
					if (toPort == null) {
						toPort = Classifier2Component.processPortForAdaptor(toSysMLPort, to.getType());
					}
					if (toPort == null) {
						toPort = getPort(toSysMLPort, to.getType());
					}
				}
				if (toPort == null) {
					toPort = new Classifier2Port(toSysMLPort.getOfClass(), toSysMLPort, null).getPort();
				}
				if (toPort.getDirection() == PortDirection.IN) {
					Port tmpPort = toPort;
					ComponentProperty tmp = to;
					MSysMLPort tmpSysMLPort = toSysMLPort;
					to = from;
					toPort = fromPort;
					toSysMLPort = fromSysMLPort;
					from = tmp;
					fromPort = tmpPort;
					fromSysMLPort = tmpSysMLPort;
				}
				Link link = StructureHelper.link(from, fromPort, Classifier2Port.getPortIndex(fromSysMLPort),
						to, toPort, Classifier2Port.getPortIndex(toSysMLPort));
				result.getLink().add(link);
				if (from != null) {
					if (Classifier2Component.isAdaptor(fromSysMLPort.getOfClass())) {
						ConfigurationElement configurationElement = null;
						for (ConfigurationElement ce : from.getConfigurationElements()) {
							if (ce.getReferenceId().equals(Classifier2Component.TEMPLATE_PORT_TYPE)) {
								configurationElement = ce;
								break;
							}
						}
						if (configurationElement == null) {
						//	Instance2ComponentProperty.createTemplateTypeConfigurationElement(fromSysMLPort.getOfClass(), from, from.getType());
						}
						for (ConfigurationElement ce : from.getConfigurationElements()) {
							if (ce.getReferenceId().getValue().equals(Classifier2Component.TEMPLATE_PORT_TYPE) && ce.getConfigurableElementValue() == null) {
								DataType type = toPort.getType();
								//AssertionUtils.assumption(type != null, "Unexpected null type");
								if (type instanceof ArrayType) {
									type = ((ArrayType)type).getElementType();
									//AssertionUtils.assumption(type != null, "Unexpected null type");
								}
								if (type != null) {
									AssertionUtils.assumption(type instanceof PrimitiveType, "Unexpected type '" + type.eClass().getName() + "'");
									switch(((PrimitiveType)type).getKind()) {
										case REAL: ce.getConfigurableElementValue().setValue("double"); break;
										case BOOLEAN: ce.getConfigurableElementValue().setValue("bool"); break;
										default: ce.getConfigurableElementValue().setValue("int"); break;
									}
								}
							}
						}
					}
				}
				if (to != null) {
					if (Classifier2Component.isAdaptor(toSysMLPort.getOfClass())) {
						ConfigurationElement configurationElement = null;
						for (ConfigurationElement ce : to.getConfigurationElements()) {
							if (ce.getReferenceId().equals(Classifier2Component.TEMPLATE_PORT_TYPE)) {
								configurationElement = ce;
								break;
							}
						}
						if (configurationElement == null) {
						//	Instance2ComponentProperty.createTemplateTypeConfigurationElement(toSysMLPort.getOfClass(), to, to.getType());
						}
						for (ConfigurationElement ce : to.getConfigurationElements()) {
							if (ce.getReferenceId().getValue().equals(Classifier2Component.TEMPLATE_PORT_TYPE) && ce.getConfigurableElementValue() == null) {
								DataType type = fromPort.getType();
								//AssertionUtils.assumption(type != null, "Unexpected null type");
								if (type instanceof ArrayType) {
									type = ((ArrayType)type).getElementType();
									//AssertionUtils.assumption(type != null, "Unexpected null type");
								}
								if (type != null) {
									AssertionUtils.assumption(type instanceof PrimitiveType, "Unexpected type '" + type.eClass().getName() + "'");
									switch(((PrimitiveType)type).getKind()) {
										case REAL: ce.getConfigurableElementValue().setValue("double"); break;
										case BOOLEAN: ce.getConfigurableElementValue().setValue("bool"); break;
										default: ce.getConfigurableElementValue().setValue("int"); break;
									}
								}
							}
						}
					}
				}
			} else {
				LOGGER.logWarning(mLink, "invalid link, 'from' and/or 'to' reference is null");
			}
		}
		// PROCESS TAGS
		for (MTag tag : self.getMAllTags()) {
			Tag2Target tag2Target = new Tag2Target(tag);
			if (tag2Target.when()) {
				result.setTarget(tag2Target.map());
			}
		}
		// PROCESS ATTRIBUTES
		for (MAttribute attribute : self.getMAttributes()) {
			Parameter parameter = new Attribute2Parameter(attribute, result.getDeclarationZone()).map();
			result.getParameter().add(parameter);
		}
		if (Tag2Target.isModelicaTarget(self) && !self.getMLinks().isEmpty()) {
			Parameter parameter = DesyreFactory.eINSTANCE.createParameter();
			parameter.setName(FMU_FILE_PATH);
			PrimitiveType type = getPrimiveType(PrimitiveTypeKind.STRING, "string");
			parameter.setType(type);
			Litteral literal = DesyreFactory.eINSTANCE.createLitteral();
			parameter.setValue(literal);
			String wxURI = Tag2Target.getWxURITagValue(self);
			String splittedPath[] = Tag2Target.getSplittedPath(wxURI);
			AssertionUtils.assumption(splittedPath != null && splittedPath.length > 1, "Invalid whitebox URI '" + wxURI + "' for component '" + self.getName() + "'");
			literal.setValue(MODELICA_FMU_PATH + File.separator + splittedPath[0] + "_" + splittedPath[1] + MODELICA_FMU_EXTENSION);
			result.getParameter().add(parameter);
		}
	}

	@Override
	protected void initInheritance() {
		inherits(ModelElement2NamedElement.class);
	}

	private ComponentProperty getComponentProperty(MInstance instance) {
		ComponentProperty componentProperty = null;
		if (!(instance instanceof MSysMLPort) && !(instance instanceof MPort)) {
			componentProperty = resolveoneIn(instance, Instance2ComponentProperty.class, ComponentProperty.class);
			if (componentProperty == null) {
				componentProperty = new Instance2ComponentProperty(instance).map();
				result.getPart().add(componentProperty);
			}
		}
		return componentProperty;
	}

	private static Port getPort(MSysMLPort sysMLPort, Component component) {
		if (sysMLPort != null) {
			for (Port port : component.getPort()) {
				if (port.getName().equals(NameMapping.getName(sysMLPort))) {
					if (port.getType() == null) {
						DataType type = new Classifier2DataType(sysMLPort.getType()).mapIfNotExists();
						port.setType(type);
					}
					return port;
				}
			}
			LOGGER.logError(sysMLPort, "cannot find port in component '" + component.getName() + "'");
		} else {
			LOGGER.logError("null port in component '" + component.getName() + "'");
		}
		return null;
	}

	public static boolean samePortKind(MSysMLPort port1, Port port2) {
		AssertionUtils.assumption(port1 != null && port2 != null, "Invalid null port argument");
		return port2.getKind() == SysMLPort2Port.getPortKind(port1);
	}

	public static Set<MStereotype> getAllStereotypesAndTheirGeneralizations(MModelElement modelElement) {
		AssertionUtils.assumption(modelElement != null, "Invalid null model element argument");
		Set<MStereotype> allStereotypes = componentToAllStereotypesAndTheirGeneralizationsMap.get(modelElement);
		if (allStereotypes == null) {
			ModelUtils.setLogger(LOGGER);
			allStereotypes = ModelUtils.getAllStereotypesAndTheirGeneralizations(modelElement);
			componentToAllStereotypesAndTheirGeneralizationsMap.put(modelElement, allStereotypes);
		}
		return allStereotypes;
	}

	public static Port processPortForAdaptor(MSysMLPort port, Component component) {
		if (port != null && component != null) {
			MClassifier classifier = port.getOfClass();
			if (isAdaptor(classifier)) {
				if (!port.getPortDirection().equals(SysMLPort2Port.IN)) {
					Port outputPort = null;
					for (Port desyrePort : component.getPort()) {
						if (desyrePort.getDirection() == PortDirection.OUT) {
							if (outputPort == null) {
								outputPort = desyrePort;
							} else {
								LOGGER.logError(classifier, "Invalid target for adaptor: corresponding blackbox component has multiple output ports");
							}
						}
					}
					if (outputPort == null) {
						LOGGER.logError(classifier, "Invalid target for adaptor: corresponding blackbox component has no output port");
					} else {
						return outputPort;
					}
				} else {
					// if SysML port is an IN port, return the input port of the DESYRE target component that implements the replicator
					for (Port desyrePort : component.getPort()) {
						if (desyrePort.getDirection() == PortDirection.IN) {
							return desyrePort;
						}
					}
					LOGGER.logError(classifier, "Invalid adaptor: no input port found");
				}
			}
		}
		return null;
	}

	public static Port processPortForReplicator(MSysMLPort port, Component component) {
		if (port != null && component != null) {
			MClassifier classifier = port.getOfClass();
			if (isReplicator(classifier)) {
				if (!port.getPortDirection().equals(SysMLPort2Port.IN)) {
					Port outputPort = null;
					for (Port desyrePort : component.getPort()) {
						if (desyrePort.getDirection() == PortDirection.OUT) {
							if (outputPort == null) {
								outputPort = desyrePort;
							} else {
								LOGGER.logError(classifier, "Invalid target for replicator: corresponding blackbox component has multiple output ports");
							}
						}
					}
					if (outputPort == null) {
						LOGGER.logError(classifier, "Invalid target for replicator: corresponding blackbox component has no output port");
					} else {
						return outputPort;
					}
				} else {
					// if SysML port is an IN port, return the input port of the DESYRE target component that implements the replicator
					for (Port desyrePort : component.getPort()) {
						if (desyrePort.getDirection() == PortDirection.IN) {
							return desyrePort;
						}
					}
					LOGGER.logError(classifier, "Invalid replicator: no input port found");
				}
			}
		}
		return null;
	}

	public static Port processPortForMulticast(MSysMLPort port, Component component) {
		if (port != null) {
			MClassifier classifier = port.getOfClass();
			if (isMulticast(classifier)) {
				// if output port has same MoCC than input port, replace multicast's output port with multicast's input port
				// this is a workaround, because the multicast is currently implemented in DESYRE with a single thickconnector
				Port inputPort = null;
				for (Port desyrePort : component.getPort()) {
					if (desyrePort.getDirection() == PortDirection.IN) {
						inputPort = desyrePort;
						break;
					}
				}
				if (inputPort == null) {
					LOGGER.logError(classifier, "Invalid multicast: no input port found");
				} else if (port.getPortDirection().equals(SysMLPort2Port.IN)) {
					return inputPort;
				} else if (samePortKind(port, inputPort)) {
					return inputPort;
				} else {
					AssertionUtils.assumption(port.getOfClass() != null, "SysML port '" + port.getName() + "' is not contained in a class");
					AssertionUtils.assumption(SysMLPort2Port.getPortKind(port) == PortKind.MODELICA_FMI, "Unexpected link to SysML port '" + port.getName() + "' of class '" + port.getOfClass().getName() + "'");
					Port outputPort = null;
					for (Port desyrePort : component.getPort()) {
						if (desyrePort.getDirection() == PortDirection.OUT) {
							if (outputPort == null) {
								outputPort = desyrePort;
							} else {
								LOGGER.logError(classifier, "Invalid target for multicast: corresponding blackbox component has multiple output ports");
							}
						}
					}
					if (outputPort == null) {
						LOGGER.logError(classifier, "Invalid target for multicast: corresponding blackbox component has no output port");
					} else {
						return outputPort;
					}
				}
			}
		}
		return null;
	}

	public static boolean isMoCCStereotype(MStereotype stereotype) {
		AssertionUtils.assumption(stereotype != null, "Unexpected null stereotype argument");
		if (stereotype.getName().equals(MOCC_STEREOTYPE_NAME)) {
			if (moccStereotype == null) {
				moccStereotype = stereotype;
			}
			return true;
		}
		Map<MClassifier,List<MClassifier>> allGeneralizations = ModelUtils.getAllGeneralizations(stereotype);
		List<MClassifier> generalizations = allGeneralizations.get(stereotype);
		if (moccStereotype == null) {
			for (MClassifier classifier : generalizations) {
				if (classifier instanceof MStereotype && classifier.getName().equals(MOCC_STEREOTYPE_NAME)) {
					moccStereotype = (MStereotype)classifier;
					return true;
				}
			}
			return false;
		}
		return generalizations.contains(moccStereotype);
	}

	public static boolean isAdaptor(MClassifier classifier) {
		return isStereotypedClassifier(classifier, ADAPTOR_STEREOTYPE_NAME, adaptorStereotype);
	}

	public static boolean isMulticast(MClassifier classifier) {
		return isStereotypedClassifier(classifier, MULTICAST_STEREOTYPE_NAME, multicastStereotype);
	}

	public static boolean isReplicator(MClassifier classifier) {
		return isStereotypedClassifier(classifier, REPLICATOR_STEREOTYPE_NAME, replicatorStereotype);
	}

	private static boolean isStereotypedClassifier(MClassifier classifier, String referenceStereotypeName, MStereotype referenceStereotype) {
		AssertionUtils.assumption(classifier != null, "Unexpected null classifier argument");
		Set<MStereotype> allStereotypes = getAllStereotypesAndTheirGeneralizations(classifier);
		AssertionUtils.assumption(allStereotypes != null, "Unexpected null stereotype set for component '" + classifier.getName() + "'");
		if (referenceStereotype == null) {
			for (MStereotype stereotype : allStereotypes) {
				if (stereotype.getName().equals(referenceStereotypeName)) {
					referenceStereotype = stereotype;
					return true;
				}
			}
			return false;
		}
		return allStereotypes.contains(referenceStereotype);
	}

	public PrimitiveType getPrimiveType(PrimitiveTypeKind kind, String name) {
		System system = resolveoneIn(self.getMProject(), Project2System.class, System.class);
		for (ReusableElement element : system.getDeclared()) {
			if (element instanceof PrimitiveType) {
				if (((PrimitiveType)element).getKind() == kind) {
					return (PrimitiveType)element;
				}
			}
		}
		PrimitiveType type = DesyreFactory.eINSTANCE.createPrimitiveType();
		system.getDeclared().add(type);
		type.setKind(kind);
		type.setName(name);
		return type;
	}
}
