
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import com.eu.ales.desyre.model.desyre.DataType;
import com.eu.ales.desyre.model.desyre.DeclarationZone;
import com.eu.ales.desyre.model.desyre.PrimitiveType;
import com.eu.ales.desyre.model.desyre.System;
import com.eu.ales.modeltransformation.impl.DisjointMappingImpl;
import com.eu.ales.rpyplugins.MClassifier;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;

public class Classifier2DataType extends DisjointMappingImpl<MClassifier, DataType> {
	private static final Logger LOGGER = Logger.getLogger(Classifier2DataType.class);

	public Classifier2DataType(MClassifier self) {
		super(self);
	}

	@Override
	protected void initDisjuction() {
		disjoint(Type2PrimitiveType.class);
		disjoint(Classifier2UnsupportedType.class);
	}

	public DataType mapIfNotExists() {
		return mapIfNotExists(null);
	}

	public DataType mapIfNotExists(DeclarationZone declarationZone) {
		DataType type = resolveoneIn(self, Type2PrimitiveType.class, PrimitiveType.class);
		if (type == null) {
			type = resolveoneIn(self, Classifier2UnsupportedType.class, PrimitiveType.class);
			if (type == null) {
				type = new Classifier2DataType(self).map();
				LOGGER.logTrace(self, "creating DESYRE type: " + ((Object)self));
				if (declarationZone == null) {
					declarationZone = resolveoneIn(self.getMProject(), Project2System.class, System.class);
				}
				AssertionUtils.assumption(declarationZone != null, "Unexpected null declaration zone");
				declarationZone.getDeclared().add(type);
			}
		}
		return type;
	}
}
