
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.eu.ales.desyre.model.desyre.ArrayType;
import com.eu.ales.desyre.model.desyre.DataType;
import com.eu.ales.desyre.model.desyre.DeclarationZone;
import com.eu.ales.desyre.model.desyre.DesyreFactory;
import com.eu.ales.desyre.model.desyre.Port;
import com.eu.ales.desyre.model.desyre.PortDirection;
import com.eu.ales.desyre.model.desyre.System;
import com.eu.ales.modeltransformation.impl.MappingImpl;
import com.eu.ales.rpyplugins.MClassifier;
import com.eu.ales.rpyplugins.MSysMLPort;
import com.eu.ales.rpyplugins.MTag;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;

public class Classifier2Port extends MappingImpl<MClassifier, Port> {
	private static final Logger LOGGER = Logger.getLogger(Classifier2Port.class);
	public static final String MODELICA_FMI_OUT_PORT_NAME = "outputPort";
	public static final String MODELICA_FMI_IN_PORT_NAME = "inputPort";
	public static final String SIMULINK_OUT_PORT_NAME = "outputPort";
	public static final String SIMULINK_IN_PORT_NAME = "inputPort";

	private static final Map<MClassifier, Integer> inPortIndexMap = new HashMap<MClassifier, Integer>();
	private static final Map<MClassifier, Integer> outPortIndexMap = new HashMap<MClassifier, Integer>();
	private static final Map<MSysMLPort, Integer> portIndexMapping = new HashMap<MSysMLPort, Integer>();

	private MSysMLPort sysMLPort;
	private DeclarationZone desyreDeclarationZone;
	private String target;
	private PortDirection portDirection;

	private Classifier2Port(MClassifier self) {
		super(self);
		this.sysMLPort = null;
		this.desyreDeclarationZone = null;
		this.target = null;
		this.portDirection = null;
		inPortIndexMap.put(self, 0);
		outPortIndexMap.put(self, 0);
	}

	public Classifier2Port(MClassifier self, MSysMLPort sysMLPort, DeclarationZone desyreDeclarationZone) {
		this(self);
		this.sysMLPort = sysMLPort;
		this.desyreDeclarationZone = desyreDeclarationZone;
		this.target = Tag2Target.isModelicaTarget(self) ? null : Tag2Target.isSimulinkTarget(self) ? Tag2Target.SIMULINK_TARGET : null;
		this.portDirection = sysMLPort.getPortDirection().equals(SysMLPort2Port.IN) ? PortDirection.IN : sysMLPort.getPortDirection().equals(SysMLPort2Port.OUT) ? PortDirection.OUT : null;
		AssertionUtils.assumption(this.sysMLPort != null, "Invalid null SysMLPort argument");
		LOGGER.logTrace(self, "Unexpected null declaration zone");
	}

	@Override
	public void populate() {
		AssertionUtils.assumption(target != null, "Unexpected null target");
		DataType type = new Classifier2DataType(sysMLPort.getType()).mapIfNotExists(desyreDeclarationZone);
		if (target == Tag2Target.SIMULINK_TARGET) {
			DataType elementType = type;
			type = DesyreFactory.eINSTANCE.createArrayType();
			((ArrayType)type).setElementType(elementType);
			if (desyreDeclarationZone != null) {
				desyreDeclarationZone.getDeclared().add(type);
			} else {
				System system = resolveoneIn(self.getMProject(), Project2System.class, System.class);
				system.getDeclared().add(type);
			}
			if (portDirection == PortDirection.IN) {
				// TODO: fix this
				((ArrayType)type).setSize(simulinkInPortIndex.keySet().size());
				result.setSize(simulinkInPortIndex.keySet().size());
			} else {
				// TODO: fix this
				((ArrayType)type).setSize(simulinkOutPortIndex.keySet().size());
				result.setSize(simulinkOutPortIndex.keySet().size());
			}
		}
		result.setType(type);
		result.setDirection(PortDirection.IN);
		result.setName(target == Tag2Target.MODELICA_TARGET ? MODELICA_FMI_IN_PORT_NAME : SIMULINK_IN_PORT_NAME);
		if (portDirection == null) {
			LOGGER.logError(self, "invalid null port direction in port '" + sysMLPort.getName() + "'");
		} else if (portDirection == PortDirection.IN) {
			result.setDirection(PortDirection.IN);
			result.setName(target == Tag2Target.MODELICA_TARGET ? MODELICA_FMI_IN_PORT_NAME : SIMULINK_IN_PORT_NAME);
		} else if (portDirection == PortDirection.OUT) {
			result.setDirection(PortDirection.OUT);
			result.setName(target == Tag2Target.MODELICA_TARGET ? MODELICA_FMI_OUT_PORT_NAME : SIMULINK_OUT_PORT_NAME);
		} else {
			LOGGER.logError(self, "invalid port direction kind '" + portDirection + "' in port '" + sysMLPort.getName() + "'");
		}
		for (MTag tag : sysMLPort.getMAllTags()) {
			Tag2Target tag2Target = new Tag2Target(tag);
			if (tag2Target.when()) {
				result.setTarget(tag2Target.map());
			}
		}
		result.setKind(SysMLPort2Port.getPortKind(sysMLPort));
	}

	public Port mapIfNotExists() {
		Port port = null;
		Collection<Port> ports = null;
		if (target != null && (target == Tag2Target.SIMULINK_TARGET || target == Tag2Target.MODELICA_TARGET && !self.getMLinks().isEmpty())) {
			if (portDirection == PortDirection.IN) {
				ports = resolveIn(self, Classifier2Port.class, Port.class);
				for (Port p : ports) {
					if (p.getDirection() == PortDirection.IN) {
						port = p;
					}
				}
				if (port == null) {
					port = map();
					Integer index = inPortIndexMap.get(self);
					AssertionUtils.assumption(index != null, "Unexpected null inport index");
					portIndexMapping.put(sysMLPort, index);
					inPortIndexMap.put(self, (index+1));
				}
			} else if (target == Tag2Target.SIMULINK_TARGET || target == Tag2Target.MODELICA_TARGET && SysMLPort2Port.isSysMLPortConnected(sysMLPort)) {
				ports = resolveIn(self, Classifier2Port.class, Port.class);
				for (Port p : ports) {
					if (p.getDirection() != PortDirection.IN) {
						port = p;
					}
				}
				if (port == null) {
					port = map();
					Integer index = outPortIndexMap.get(self);
					AssertionUtils.assumption(index != null, "Unexpected null outport index");
					portIndexMapping.put(sysMLPort, index);
					outPortIndexMap.put(self, (index+1));
				}
			}
		}
		return port;
	}

	private static final Map<String,Integer> simulinkInPortIndex = new HashMap<String, Integer>() {{
		put("V_Load",0);
		put("I_Load",1);
		put("T_Env",2);
		put("V_Gen",3);
		put("I_Gen",4);
	}};
	private static final Map<String,Integer> simulinkOutPortIndex = new HashMap<String, Integer>() {{
		put("T_Load",0);
		put("T_Gen",1);
	}};
	public static int getPortIndex(MSysMLPort sysMLPort) {
		Integer index = null;
		boolean isSimulinkTarget = Tag2Target.isSimulinkTarget(sysMLPort.getOfClass());
		AssertionUtils.assumption(sysMLPort != null, "Unexpected null SysML port");
		if (sysMLPort.getPortDirection().equals(SysMLPort2Port.IN)) {
			if (isSimulinkTarget) {
				index = simulinkInPortIndex.get(sysMLPort.getName());
				AssertionUtils.assumption(index != null, "Cannot find index of port '" + sysMLPort.getName() + "' for component '" + sysMLPort.getOfClass().getName() + "'"); 
			} else {
				index = inPortIndexMap.get(sysMLPort);
			}
		} else if (sysMLPort.getPortDirection().equals(SysMLPort2Port.OUT)) {
			if (isSimulinkTarget) {
				index = simulinkOutPortIndex.get(sysMLPort.getName());
				AssertionUtils.assumption(index != null, "Cannot find index of port '" + sysMLPort.getName() + "' for component '" + sysMLPort.getOfClass().getName() + "'"); 
			} else {
				index = outPortIndexMap.get(sysMLPort);
			}
		} else {
			LOGGER.logError(sysMLPort, "Invalid direction '" + sysMLPort.getPortDirection() + "' for SysML port '" + sysMLPort.getName() + "'");
		}
		return index != null ? index : 0;
	}

	public Port getPort() {
		Collection<Port> ports = null;
		if (target != null) {
			if (portDirection == PortDirection.IN) {
				ports = resolveIn(self, Classifier2Port.class, Port.class);
				for (Port port : ports) {
					if (port.getDirection() == PortDirection.IN) {
						return port;
					}
				}
			} else if (target == Tag2Target.SIMULINK_TARGET || target == Tag2Target.MODELICA_TARGET && SysMLPort2Port.isSysMLPortConnected(sysMLPort)) {
				ports = resolveIn(self, Classifier2Port.class, Port.class);
				for (Port port : ports) {
					if (port.getDirection() != PortDirection.IN) {
						return port;
					}
				}
			}
		}
		System system = resolveoneIn(self.getMProject(), Project2System.class, System.class);
		Port port = new Classifier2Port(self, sysMLPort, system).mapIfNotExists();
		return port;
	}
}
