
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.eu.ales.desyre.model.desyre.ArrayType;
import com.eu.ales.desyre.model.desyre.Component;
import com.eu.ales.desyre.model.desyre.ComponentProperty;
import com.eu.ales.desyre.model.desyre.ConfigurationElement;
import com.eu.ales.desyre.model.desyre.DataType;
import com.eu.ales.desyre.model.desyre.DesyreFactory;
import com.eu.ales.desyre.model.desyre.Library;
import com.eu.ales.desyre.model.desyre.Litteral;
import com.eu.ales.desyre.model.desyre.Parameter;
import com.eu.ales.desyre.model.desyre.PrimitiveType;
import com.eu.ales.desyre.model.desyre.ReusableElement;
import com.eu.ales.desyre.model.desyre.System;
import com.eu.ales.desyre.model.desyre.View;
import com.eu.ales.modeltransformation.impl.DerivedMappingImpl;
import com.eu.ales.rpyplugins.MAttribute;
import com.eu.ales.rpyplugins.MClass;
import com.eu.ales.rpyplugins.MClassifier;
import com.eu.ales.rpyplugins.MInstance;
import com.eu.ales.rpyplugins.MModelElement;
import com.eu.ales.rpyplugins.MSysMLPort;
import com.eu.ales.rpyplugins.MTag;
import com.eu.ales.rpyplugins.MType;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;
import com.eu.ales.utils.AssumptionViolation;

public class Instance2ComponentProperty extends DerivedMappingImpl<MInstance, ComponentProperty> {
	private static final Logger LOGGER = Logger.getLogger(Instance2ComponentProperty.class);

	public Instance2ComponentProperty(MInstance self) {
		super(self);
	}

	@Override
	public void populate() 
	{
		LOGGER.logTrace("translating instance '" + self.getName() + "' of type '" + self.getOtherClass().getFullPathName() + "'");
		Component component = getComponent(self.getOtherClass());
		if( component == null ) {
			LOGGER.logError("cannot find component '" + self.getName() + "' of type '" + self.getOtherClass().getFullPathName() + "'");
		}
		result.setType(component);
		View view = resolveoneIn(self.getOtherClass(), Classifier2View.class, View.class);
		result.setView(!component.getView().isEmpty() ? component.getView().get(0) : view);
		// TODO: multiplicity has to be taken into account
		result.setSize(1);
		List<String> attrNames = new ArrayList<String>();
		// first process all attributes specified in the instance
		for (MAttribute attribute : self.getMAttributeValues()) {
			attrNames.add(attribute.getName());
			ConfigurationElement configurationElement = new Attribute2ConfigurationElement(attribute).map();
			result.getConfigurationElements().add(configurationElement);
		}
		// for those attributes not specified in the instance copy the corresponding attribute from the defining class
		AssertionUtils.assumption(self.getOtherClass() != null, "Missing type for instance '" + self.getName() + "'");
		for (MAttribute attribute : self.getOtherClass().getMAttributes()) {
			if (!attrNames.contains(attribute.getName())) {
				ConfigurationElement configurationElement = Attribute2ConfigurationElement.createConfigurationElement(attribute);
				result.getConfigurationElements().add(configurationElement);
			}
		}
		createTemplateTypeConfigurationElement(self.getOtherClass(), result, component);
	}

	public void createTemplateTypeConfigurationElement(MClassifier classifier, ComponentProperty result, Component component) {
		if (Classifier2Component.isAdaptor(classifier)) {
			boolean found = false;
			for (ConfigurationElement ce : result.getConfigurationElements()) {
				if (ce.getReferenceId().equals(Classifier2Component.TEMPLATE_PORT_TYPE)) {
					found = true;
					break;
				}
			}
			if (!found) {
				ConfigurationElement configurationElement = DesyreFactory.eINSTANCE.createConfigurationElement();
				result.getConfigurationElements().add(configurationElement);
				Litteral lit1 = DesyreFactory.eINSTANCE.createLitteral();
				configurationElement.setReferenceId(lit1);
				lit1.setValue(Classifier2Component.TEMPLATE_PORT_TYPE);
				Litteral lit2 = DesyreFactory.eINSTANCE.createLitteral();
				configurationElement.setConfigurableElementValue(lit2);
				AssertionUtils.assumption(!component.getPort().isEmpty(), "Unexpected component without ports '" + component.getName() + "'");
				DataType type = component.getPort().get(0).getType();
				//AssertionUtils.assumption(type != null, "Unexpected null type");
				if (type instanceof ArrayType) {
					type = ((ArrayType)type).getElementType();
					//AssertionUtils.assumption(type != null, "Unexpected null type");
				}
				if (type != null) {
					AssertionUtils.assumption(type instanceof PrimitiveType, "Unexpected type '" + type.eClass().getName() + "'");
					switch(((PrimitiveType)type).getKind()) {
					case REAL: lit2.setValue("double"); break;
					case BOOLEAN: lit2.setValue("bool"); break;
					default: lit2.setValue("int"); break;
					}
				} else {
					MType t = null;
					PrimitiveType primitiveType = null;
					System system = resolveoneIn(self.getMProject(), Project2System.class, System.class);
					for (MModelElement me : classifier.getMNestedElements()) {
						if (me instanceof MSysMLPort) {
							MClassifier c = ((MSysMLPort)me).getType();
							if (c instanceof MType) {
								primitiveType = resolveoneIn(c, Type2PrimitiveType.class, PrimitiveType.class);
								if (primitiveType == null) {
									primitiveType = new Type2PrimitiveType((MType)c).map();
									system.getDeclared().add(primitiveType);
								}
								switch(primitiveType.getKind()) {
								case REAL: lit2.setValue("double"); break;
								case BOOLEAN: lit2.setValue("bool"); break;
								default: lit2.setValue("int"); break;
								}
							}
						}
					}
				}
			}
		}
	}
	@Override
	protected void initInheritance() {
		inherits(ModelElement2NamedElement.class);
	}

	private Component getComponent(MClassifier classifier) {
		AssertionUtils.assumption(classifier != null, "Invalid null classifier argument");
		AssertionUtils.assumption(classifier instanceof MClass, "classifier of type '" + classifier.eClass().getName() + "' is not supported for transalation to DESYRE component");
		Component component = getComponentFromDesyreAdaptorLibrary(classifier);
		if (component == null) {
			// if component not found in DESYRE adaptor library, create component;
			// assume that all components are added to the system, so that the DESYRE 
			// code generator will generate the file system structure for such components.
			component = resolveoneIn(classifier, Classifier2Component.class, Component.class);
			if (component == null) {
				System system = resolveoneIn(self.getMProject(), Project2System.class, System.class);
				component = new Classifier2Component(classifier, system.getName()).map();
			}
			return component;
		} else {
			// set the parameter's default value for components read from the DESYRE library, 
			// for which no mapping is invoked
			for (MAttribute attribute : classifier.getMAttributes()) {
				boolean found = false;
				for (Parameter parameter : component.getParameter()) {
					if (attribute.getName().equals(parameter.getName())) {
						Attribute2Parameter.setParameter(attribute, parameter, true /*check types*/);
						found = true;
					}
				}
				if (!found) {
					LOGGER.logError(classifier, "unable to find parameter '" + attribute.getName() + "' in DESYRE target component");
				}
			}
		}
		return component;
	}

	private Component getComponentFromDesyreAdaptorLibrary(MClassifier classifier) {
		String desyreLibraryName = null;
		String desyreComponentName = NameMapping.getName(classifier);
		if (Classifier2Component.isAdaptor(classifier)) {
			System system = resolveoneIn(self.getMProject(), Project2System.class, System.class);
			List<MTag> tags = classifier.getMAllTags();
			for (MTag tag : tags) {
				// look for blackbox URI tag, where the correct DESYRE LIB and IP name of the adaptor component is expected to be specified
				if (tag.getName().equals(Tag2Target.BLACKBOX_URI)) {
					String authority = Tag2Target.getAuthority(tag.getValue());
					if (authority != null && authority.equals(Tag2Target.DESYRE_TARGET)) {
						// take DESYRE LIB and IP name from the value of the blackbox URI tag
						String splittedPath[] = Tag2Target.getSplittedPath(tag.getValue());
						if (splittedPath.length == 2) {
							desyreLibraryName = splittedPath[0];
							desyreComponentName = splittedPath[1];
							// look for component inside DESYRE adaptor library
							EList<Library> libraries = system.getLibraries();
							for (Library lib : libraries) {
								// first look for name-matching DESYRE library
								if (lib.getName().equals(desyreLibraryName)) {
									for (ReusableElement element : lib.getDeclared()) {
										// when DESYRE library found, look for name-matching component
										if (element instanceof Component && element.getName().equals(desyreComponentName)) {
											return (Component)element;
										}
									}
									break;
								}
							}
						} else {
							LOGGER.logError(classifier, "unexpected '" + Tag2Target.BLACKBOX_URI + "'='" + tag.getValue() + "': '" + Tag2Target.DESYRE_TARGET + Tag2Target.AUTHORITY_SEPARATOR + "Lib" + Tag2Target.URI_PATH_SEPARATOR + "Ip' expected");
						}
					} else {
						LOGGER.logError(classifier, "unexpected '" + Tag2Target.BLACKBOX_URI + "' authority '" + authority + "'");
					}
					break;
				}
			}
			LOGGER.logError(classifier, "cannot find corresponding '" + Classifier2Component.ADAPTOR_STEREOTYPE_NAME + "' in DESYRE libraries");
		}
		return null;
	}
}

