
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import com.eu.ales.desyre.model.desyre.DeclarationZone;
import com.eu.ales.desyre.model.desyre.DesyreFactory;
import com.eu.ales.desyre.model.desyre.DesyrePackage;
import com.eu.ales.desyre.model.desyre.Library;
import com.eu.ales.desyre.model.desyre.System;
import com.eu.ales.desyre.model.desyre.util.DesyreResourceFactoryImpl;
import com.eu.ales.desyre.model.desyre.util.DesyreUtils;
import com.eu.ales.modeltransformation.Transformation;
import com.eu.ales.modeltransformation.impl.MappingHelper;
import com.eu.ales.rpyplugins.MProject;
import com.eu.ales.rpyplugins.RpypluginsPackage;
import com.eu.ales.rpyplugins.utils.FileManager;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.rpyplugins.utils.Serialization;
import com.eu.ales.utils.AssertionUtils;

public class MetaII2DesyreTransformation implements Transformation<MProject, System> {
	private Map<String, Object> context = new HashMap<String, Object>();
	private static Map<String, DeclarationZone> desyreDeclarationZoneMap = new HashMap<String, DeclarationZone>();
	private static List<Library> newDesyreLibraries = new ArrayList<Library>();

	@Override
	public System execute(MProject input) {
		// register the factories
		MappingHelper.getInstance().addMetamodel(RpypluginsPackage.eINSTANCE);
		MappingHelper.getInstance().addMetamodel(DesyrePackage.eINSTANCE);

		System desyreSystem = new Project2System(input, desyreDeclarationZoneMap.values()).map();

		List<Library> libraries = desyreSystem.getLibraries();
		for (DeclarationZone library : desyreDeclarationZoneMap.values()) {
			if (library instanceof Library && !libraries.contains(library)) {
				libraries.add((Library)library);
			}
		}

		createResources(desyreSystem);

		return desyreSystem;
	}

	@Override
	public void setContextElement(String key, Object value) {
		context.put(key, value);
	}

	@Override
	public Object getContextElement(String key) {
		return context.get(key);
	}

	public static DeclarationZone getDesyreDeclarationZone(String name) {
		AssertionUtils.assumption(name != null && !name.isEmpty(), "Invalid null or empty library name");
		DeclarationZone declarationZone = desyreDeclarationZoneMap.get(name);
		if (declarationZone == null) {
			declarationZone = DesyreFactory.eINSTANCE.createLibrary();
			declarationZone.setName(name);
			desyreDeclarationZoneMap.put(name, declarationZone);
			newDesyreLibraries.add((Library)declarationZone);
		}
		return declarationZone;
	}

	public static void setDesyreDeclarationZone(DeclarationZone declarationZone) {
		if (declarationZone != null && declarationZone.getName() != null && !declarationZone.getName().isEmpty()) {
			desyreDeclarationZoneMap.put(declarationZone.getName(), declarationZone);
		}
	}

	public static List<Library> getNewDesyreLibraries() {
		return newDesyreLibraries;
	}

	private static void createResources(System desyreSystem) {

		/* Create a resource set to hold the resources. */
		ResourceSet resourceSet = new ResourceSetImpl();
		
		/*
		 * Register the appropriate resource factory to handle all file
		 * extensions.
		 */
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				Resource.Factory.Registry.DEFAULT_EXTENSION, new DesyreResourceFactoryImpl());
		
		resourceSet.getPackageRegistry().put(DesyrePackage.eNS_URI, DesyrePackage.eINSTANCE);

		//File desyreFile = FileManager.getXmiFile(desyreModelFileName);
		//URI modelURI = URI.createFileURI(desyreFile.getAbsolutePath());
		URI modelURI = FileManager.createDesyreModelURI(FileManager.DESYRE_RESOURCE_FILE_NAME);
		Resource desyreResource = resourceSet.createResource(modelURI);	
		desyreResource.getContents().add(desyreSystem);

		for (Library library : desyreSystem.getLibraries()) {
			if (library.eResource() == null) {
				//File libraryFile = FileManager.getXmiFile(library.getName());
				//URI libraryURI = URI.createFileURI(libraryFile.getAbsolutePath());
				URI libraryURI = FileManager.createDesyreModelURI(library.getName());
				Resource libraryResource = resourceSet.createResource(libraryURI);	
				libraryResource.getContents().add(library);
			}
		}
	}

	public static MProject loadMetaIIProject(Logger logger) {
		URI uri = FileManager.createMetaIIProjectURI(FileManager.METAII_RESOURCE_FILE_NAME);
		File xmiFile = new File(uri.toFileString());
		MProject project = null;
		EObject model = null;
		try {
			model = Serialization.loadRootFromFile(xmiFile, RpypluginsPackage.eNS_URI, RpypluginsPackage.eINSTANCE);
			AssertionUtils.assumption(model instanceof MProject, "Invalid model element object '" + model.eClass().getName()	+ "' loaded from file '" + xmiFile.getAbsolutePath() + "': MProject expected");

			project = (MProject)model;
			//// print out Rhapsody model
			//logger.log("Printing Rhapsody model");
			//logger.printOut(project);
		} catch (IOException e) {
			logger.logError("IOException while loading model from file '" + xmiFile.getAbsolutePath() + "': " + e.getMessage());
		} catch (ClassCastException e) {
			logger.logError("Invalid casting '" + model.eClass().getName() + "' to 'MProject' while loading model from file '" + xmiFile.getAbsolutePath() + "': " + e.getMessage());
		}
		return project;
	}

	public static void loadDesyreLibraries(Logger logger) {
		// load DESYRE libraries
		for (String libraryName : FileManager.DESYRE_LIBRARY_NAMES) {
			//String libURI = com.eu.ales.meta.sysml2desyre.libraries.Activator.DESYRE_LIB_PATH + File.separator + libraryName + ".xmi";
			//logger.log("Loading DESYRE library from file '" + libURI + "'");
			//URI metaIILibURI = URI.createPlatformPluginURI(libURI, false);
			//Library metaIILib = DesyreUtils.getDesyreLibrary(metaIILibURI);
			//MetaII2DesyreTransformation.setDesyreLibrary(metaIILib);

			URI libURI = FileManager.createDesyreWorkspaceURI(libraryName);
			// loading DESYRE library
			logger.logInfo("Loading DESYRE library from file '" + libURI.toFileString() + "'");
			Library metaIILib = DesyreUtils.getDesyreLibrary(libURI);
			MetaII2DesyreTransformation.setDesyreDeclarationZone(metaIILib);
		}
	}

	public static void saveDesyreSystemAndLibraries(com.eu.ales.desyre.model.desyre.System desyreSystem, Logger logger) {
		FileManager.saveDesyreModelToFile(desyreSystem, getNewDesyreLibraries(), DesyrePackage.eNS_URI, DesyrePackage.eINSTANCE, logger);
	}
}
