
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import com.eu.ales.desyre.model.desyre.ArrayType;
import com.eu.ales.desyre.model.desyre.Component;
import com.eu.ales.desyre.model.desyre.ComponentProperty;
import com.eu.ales.desyre.model.desyre.ConfigurationElement;
import com.eu.ales.desyre.model.desyre.DataType;
import com.eu.ales.desyre.model.desyre.Link;
import com.eu.ales.desyre.model.desyre.Port;
import com.eu.ales.desyre.model.desyre.PortDirection;
import com.eu.ales.desyre.model.desyre.PrimitiveType;
import com.eu.ales.desyre.model.desyre.View;
import com.eu.ales.desyre.model.desyre.util.StructureHelper;
import com.eu.ales.modeltransformation.impl.DerivedMappingImpl;
import com.eu.ales.rpyplugins.MInstance;
import com.eu.ales.rpyplugins.MLink;
import com.eu.ales.rpyplugins.MPackage;
import com.eu.ales.rpyplugins.MPort;
import com.eu.ales.rpyplugins.MSysMLPort;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.utils.AssertionUtils;
import com.sun.org.apache.bcel.internal.util.Class2HTML;

public class Package2Component extends DerivedMappingImpl<MPackage, com.eu.ales.desyre.model.desyre.Component> {
	private static final Logger LOGGER = Logger.getLogger(Package2Component.class);

	public Package2Component(MPackage self) {
		super(self);
	}

	static int p = 0;
	@Override
	public void populate() {
		// override the default name by prepending the name of all the super-packages
		String qualifiedName = NameMapping.getQualifiedName(self.getFullPathName());
		result.setName(qualifiedName);
		View view = new Package2View(self).map();
		result.getView().add(view);
		result.setUri(view.getName() + "://" + result.getName());
		// PROCESS LINKS
		for (MLink mLink : self.getMLinks()) {
			Type2PrimitiveType.resolveInheritedDataType(mLink);
			if (mLink.getFrom() != null && mLink.getTo() != null) {
				ComponentProperty from = getComponentProperty(mLink.getFrom());
				ComponentProperty to = getComponentProperty(mLink.getTo());
				if (from != null && from.getName().contains("Thermal") || to != null && to.getName().contains("Thermal")) {
					p++;
					LOGGER.logError("COUNTER " + p);
				}
				Port fromPort = null;
				MSysMLPort fromSysMLPort = mLink.getFromSysMLPort();
				if (fromSysMLPort == null && mLink.getFrom() instanceof MSysMLPort) {
					// this link is a hierarchical connection
					fromSysMLPort = (MSysMLPort)mLink.getFrom();
					fromPort = getPort(fromSysMLPort, result);
				} else {
					if (from != null) {
						fromPort = Classifier2Component.processPortForReplicator(fromSysMLPort, from.getType());
					}
					if (fromPort == null) {
						fromPort = Classifier2Component.processPortForMulticast(fromSysMLPort, from.getType());
					}
					if (fromPort == null) {
						fromPort = Classifier2Component.processPortForAdaptor(fromSysMLPort, from.getType());
					}
					if (fromPort == null) {
						fromPort = getPort(fromSysMLPort, from.getType());
					}
				}
				if (fromPort == null) {
					fromPort = new Classifier2Port(fromSysMLPort.getOfClass(), fromSysMLPort, null).getPort();
				}
				Port toPort = null;
				MSysMLPort toSysMLPort = mLink.getToSysMLPort();
				if (toSysMLPort == null && mLink.getTo() instanceof MSysMLPort) {
					// this link is a hierarchical connection
					toSysMLPort = (MSysMLPort)mLink.getTo();
					toPort = getPort(toSysMLPort, result);
				} else {
					if (to != null) {
						toPort = Classifier2Component.processPortForReplicator(toSysMLPort, to.getType());
					}
					if (toPort == null) {
						toPort = Classifier2Component.processPortForMulticast(toSysMLPort, to.getType());
					}
					if (toPort == null) {
						toPort = Classifier2Component.processPortForAdaptor(toSysMLPort, to.getType());
					}
					if (toPort == null) {
						toPort = getPort(toSysMLPort, to.getType());
					}
				}
				if (toPort == null) {
					toPort = new Classifier2Port(toSysMLPort.getOfClass(), toSysMLPort, null).getPort();
				}
				if (toPort.getDirection() == PortDirection.IN) {
					Port tmpPort = toPort;
					ComponentProperty tmp = to;
					MSysMLPort tmpSysMLPort = toSysMLPort;
					to = from;
					toPort = fromPort;
					toSysMLPort = fromSysMLPort;
					from = tmp;
					fromPort = tmpPort;
					fromSysMLPort = tmpSysMLPort;
				}
				Link link = StructureHelper.link(from, fromPort, Classifier2Port.getPortIndex(fromSysMLPort),
						to, toPort, Classifier2Port.getPortIndex(toSysMLPort));
				result.getLink().add(link);
				if (from != null) {
					if (Classifier2Component.isAdaptor(fromSysMLPort.getOfClass())) {
						ConfigurationElement configurationElement = null;
						for (ConfigurationElement ce : from.getConfigurationElements()) {
							if (ce.getReferenceId().equals(Classifier2Component.TEMPLATE_PORT_TYPE)) {
								configurationElement = ce;
								break;
							}
						}
						if (configurationElement == null) {
						//	Instance2ComponentProperty.createTemplateTypeConfigurationElement(fromSysMLPort.getOfClass(), from, from.getType());
						}
						for (ConfigurationElement ce : from.getConfigurationElements()) {
							if (ce.getReferenceId().getValue().equals(Classifier2Component.TEMPLATE_PORT_TYPE) && ce.getConfigurableElementValue() == null) {
								DataType type = toPort.getType();
								//AssertionUtils.assumption(type != null, "Unexpected null type");
								if (type instanceof ArrayType) {
									type = ((ArrayType)type).getElementType();
									//AssertionUtils.assumption(type != null, "Unexpected null type");
								}
								if (type != null) {
									AssertionUtils.assumption(type instanceof PrimitiveType, "Unexpected type '" + type.eClass().getName() + "'");
									switch(((PrimitiveType)type).getKind()) {
										case REAL: ce.getConfigurableElementValue().setValue("double"); break;
										case BOOLEAN: ce.getConfigurableElementValue().setValue("bool"); break;
										default: ce.getConfigurableElementValue().setValue("int"); break;
									}
								}
							}
						}
					}
				}
				if (to != null) {
					if (Classifier2Component.isAdaptor(toSysMLPort.getOfClass())) {
						ConfigurationElement configurationElement = null;
						for (ConfigurationElement ce : to.getConfigurationElements()) {
							if (ce.getReferenceId().equals(Classifier2Component.TEMPLATE_PORT_TYPE)) {
								configurationElement = ce;
								break;
							}
						}
						if (configurationElement == null) {
						//	Instance2ComponentProperty.createTemplateTypeConfigurationElement(toSysMLPort.getOfClass(), to, to.getType());
						}
						for (ConfigurationElement ce : to.getConfigurationElements()) {
							if (ce.getReferenceId().getValue().equals(Classifier2Component.TEMPLATE_PORT_TYPE) && ce.getConfigurableElementValue() == null) {
								DataType type = fromPort.getType();
								//AssertionUtils.assumption(type != null, "Unexpected null type");
								if (type instanceof ArrayType) {
									type = ((ArrayType)type).getElementType();
									//AssertionUtils.assumption(type != null, "Unexpected null type");
								}
								if (type != null) {
									AssertionUtils.assumption(type instanceof PrimitiveType, "Unexpected type '" + type.eClass().getName() + "'");
									switch(((PrimitiveType)type).getKind()) {
										case REAL: ce.getConfigurableElementValue().setValue("double"); break;
										case BOOLEAN: ce.getConfigurableElementValue().setValue("bool"); break;
										default: ce.getConfigurableElementValue().setValue("int"); break;
									}
								}
							}
						}
					}
				}
			} else {
				LOGGER.logWarning(mLink, "invalid link, 'from' and/or 'to' reference is null");
			}
		}
	}

	@Override
	protected void initInheritance() {
		inherits(ModelElement2NamedElement.class);
	}


	private ComponentProperty getComponentProperty(MInstance instance) {
		ComponentProperty componentProperty = null;
		if (!(instance instanceof MSysMLPort) && !(instance instanceof MPort)) {
			componentProperty = resolveoneIn(instance, Instance2ComponentProperty.class, com.eu.ales.desyre.model.desyre.ComponentProperty.class);
			if (componentProperty == null) {
				componentProperty = new Instance2ComponentProperty(instance).map();
				result.getPart().add(componentProperty);
			}
		}
		return componentProperty;
	}

	private Port getPort(MInstance instance, Component component) {
		if (instance instanceof MSysMLPort) {
			MSysMLPort sysMLPort = (MSysMLPort)instance;
			AssertionUtils.assumption(component != null, "Unexpected null component, cannot find DESYRE port corresponding to SysML port '" + sysMLPort.getName() + "'");
			if (component.getPort() != null) {
				for (com.eu.ales.desyre.model.desyre.Port port : component.getPort()) {
					if (port.getName().equals(NameMapping.getName(sysMLPort))) {
						return port;
					}
				}
			}
			LOGGER.logError("cannot find port '" + sysMLPort.getName() + "' in component '" + component.getName() + "'");
		} else if (instance == null) {
			LOGGER.logError("null port in component '" + component.getName() + "'");
		} else {
			LOGGER.logError(instance, "unexpected model element, 'MSysMLPort' expected");
		}
		return null;
	}
}
