
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;


import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import com.eu.ales.desyre.model.desyre.Component;
import com.eu.ales.desyre.model.desyre.DeclarationZone;
import com.eu.ales.desyre.model.desyre.Library;
import com.eu.ales.modeltransformation.impl.DerivedMappingImpl;
import com.eu.ales.rpyplugins.MPackage;
import com.eu.ales.rpyplugins.MProject;
import com.eu.ales.rpyplugins.utils.Logger;

public class Project2System extends DerivedMappingImpl<MProject, com.eu.ales.desyre.model.desyre.System> {
	private static final Logger logger = Logger.getLogger(Project2System.class);
	private List<DeclarationZone> libs;

	public Project2System(MProject self) {
		super(self);
		this.libs = new LinkedList<DeclarationZone>();
	}

	public Project2System(MProject self, Collection<DeclarationZone> additionalLibs) {
		this(self);
		libs.addAll(additionalLibs);
	}

	@Override
	public void populate() {
		result.setName(NameMapping.getName(self.getName()));
		MetaII2DesyreTransformation.setDesyreDeclarationZone(result);

		// As a first step, add a reference to library elements
		for (DeclarationZone lib : libs) {
			if (lib instanceof Library) {
				result.getLibraries().add((Library)lib);
			} else if (lib == null) {
				logger.logWarning("attempt at loading a null library element");
			} else {
				logger.logWarning("attempt at loading a null library element");
			}
		}
		
		
		for (MPackage pkg : self.getMPackages()) {
			if (pkg.getMLinks().size() > 0) {
				com.eu.ales.desyre.model.desyre.Component component = resolveoneIn(pkg, Package2Component.class, Component.class);
				if (component == null) {
					component = new Package2Component(pkg).map();
				}
				result.setRoot(component);
				result.getDeclared().add(component);
			} else {
				logger.logWarning("unsupported package: " + pkg.getName());
			}
		}
	}

	@Override
	protected void initInheritance() {
		inherits(ModelElement2NamedElement.class);
	}
}
