
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaii2desyre.impl;

import java.util.HashMap;
import java.util.Map;

import com.eu.ales.desyre.model.desyre.PrimitiveType;
import com.eu.ales.desyre.model.desyre.PrimitiveTypeKind;
import com.eu.ales.desyre.model.desyre.System;
import com.eu.ales.modeltransformation.impl.DerivedMappingImpl;
import com.eu.ales.rpyplugins.MLink;
import com.eu.ales.rpyplugins.MProject;
import com.eu.ales.rpyplugins.MSysMLPort;
import com.eu.ales.rpyplugins.MType;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.rpyplugins.utils.ModelUtils;
import com.eu.ales.utils.AssertionUtils;

public class Type2PrimitiveType extends DerivedMappingImpl<com.eu.ales.rpyplugins.MType, com.eu.ales.desyre.model.desyre.PrimitiveType> {
	private static final Logger LOGGER = Logger.getLogger(Type2PrimitiveType.class);;
	private static final String INHERIT_DATA_TYPE_NAME = "Inherit";
	private static final Map<String, PrimitiveTypeKind> PRIMITIVE_DATA_TYPES_MAP = new HashMap<String, PrimitiveTypeKind>(){{
		put("bool",PrimitiveTypeKind.BOOLEAN);
		put("boolean",PrimitiveTypeKind.BOOLEAN);
		put("OMBoolean",PrimitiveTypeKind.BOOLEAN);
		put("RhpBoolean",PrimitiveTypeKind.BOOLEAN);
		put("char",PrimitiveTypeKind.INTEGER);
		put("short",PrimitiveTypeKind.INTEGER);
		put("int",PrimitiveTypeKind.INTEGER);
		put("long",PrimitiveTypeKind.INTEGER);
		put("RhpInteger",PrimitiveTypeKind.INTEGER);
		put("float",PrimitiveTypeKind.REAL);
		put("double",PrimitiveTypeKind.REAL);
		put("RhpReal",PrimitiveTypeKind.REAL);
		put("string",PrimitiveTypeKind.STRING);
		put("String",PrimitiveTypeKind.STRING);
		put("OMString",PrimitiveTypeKind.STRING);
		put("RhpString",PrimitiveTypeKind.STRING);
		// the 'Inherit'  data type is not actually mapped to PrimitiveTypeKind.INTEGER, 
		// while the correct type is inherited from the port on the other end of the link
		put(INHERIT_DATA_TYPE_NAME,PrimitiveTypeKind.INTEGER);
	}};

	private PrimitiveTypeKind primitiveDataType;

	public Type2PrimitiveType(MType self) {
		super(self);
		this.primitiveDataType = PrimitiveTypeKind.INTEGER;
	}

	@Override
	public boolean when() {
		this.primitiveDataType = getPrimitiveTypeKind(self);
		return when(self);
	}

	@Override
	public void populate() {
		if (this.primitiveDataType == null) {
			LOGGER.logError("Unexpected null primitive data type kind, creating integer primitive data type");
			this.primitiveDataType = PrimitiveTypeKind.INTEGER;
		}
		result.setKind(this.primitiveDataType);
	}

	@Override
	protected void initInheritance() {
		inherits(ModelElement2NamedElement.class);
	}

	public static boolean when(MType self) {
		return getPrimitiveTypeKind(self) != null;
	}

	public static PrimitiveTypeKind getPrimitiveTypeKind(MType type) {
		return PRIMITIVE_DATA_TYPES_MAP.get(type.getName().trim());
	}

	public static void resolveInheritedDataType(MLink link) {
		AssertionUtils.assumption(link != null, "Invalid null link argument");
		MSysMLPort fromPort = link.getFromSysMLPort();
		if (fromPort == null && link.getFrom() instanceof MSysMLPort) {
			fromPort = (MSysMLPort)link.getFrom();
		} else if (fromPort == null) {
			LOGGER.logError("Invalid link '" + link.getName() + "', missing 'from' port");
			return;
		} else if (fromPort.getType() == null) {
			LOGGER.logError("Invalid link '" + link.getName() + "', missing type on 'from' port");
			return;
		}
		MSysMLPort toPort = link.getToSysMLPort();
		if (toPort == null && link.getTo() instanceof MSysMLPort) {
			toPort = (MSysMLPort)link.getTo();
		} else if (toPort == null) {
			LOGGER.logError("Invalid link '" + link.getName() + "', missing 'to' port");
			return;
		} else if (toPort.getType() == null) {
			LOGGER.logError("Invalid link '" + link.getName() + "', missing type on 'to' port");
			return;
		}
		if (fromPort.getType().getName().equals(INHERIT_DATA_TYPE_NAME)) {
			if (toPort.getType().getName().equals(INHERIT_DATA_TYPE_NAME)) {
				LOGGER.logError("Invalid link '" + link.getName() + "', both 'to' port and 'from' port have '" + INHERIT_DATA_TYPE_NAME + "' type");
			} else {
				// set 'from' port type to refer to 'to' port type
				ModelUtils.setECoreRelation(fromPort, toPort.getType(), fromPort.eClass().getEStructuralFeature("type"));
			}
		} else if(toPort.getType().getName().equals(INHERIT_DATA_TYPE_NAME)) {
			// set 'to' port type to refer to 'from' port type
			ModelUtils.setECoreRelation(toPort, fromPort.getType(), fromPort.eClass().getEStructuralFeature("type"));
		}
	}
}
