
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.metaiivalidator;

import java.io.File;
import java.io.IOException;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;

import com.eu.ales.rpyplugins.MProject;
import com.eu.ales.rpyplugins.RpypluginsPackage;
import com.eu.ales.rpyplugins.metaiivalidator.impl.MetaIIValidatorImpl;
import com.eu.ales.rpyplugins.utils.FileManager;
import com.eu.ales.rpyplugins.utils.Logger;
import com.eu.ales.rpyplugins.utils.Serialization;

public abstract class MetaIIValidator {
	public static ValidationResults validate(MProject project) {
		MetaIIValidatorImpl validator = new MetaIIValidatorImpl(project);
		return validator.validate();
	}

	public static MProject loadMetaIIProject(Logger logger) {
		URI uri = FileManager.createMetaIIProjectURI(FileManager.METAII_RESOURCE_FILE_NAME);
		File xmiFile = new File(uri.toFileString());
		MProject project = null;
		EObject model = null;
		try {
			model = Serialization.loadRootFromFile(xmiFile, RpypluginsPackage.eNS_URI, RpypluginsPackage.eINSTANCE);
			if (!(model instanceof MProject)) {
				throw new Error("Invalid model element object '" + model.eClass().getName()	+ "' loaded from file '" + xmiFile.getAbsolutePath() + "': MProject expected");
			}

			project = (MProject)model;
			//// print out Rhapsody model
			//logger.log("Printing Rhapsody model");
			//logger.printOut(project);
		} catch (IOException e) {
			logger.logError("IOException while loading model from file '" + xmiFile.getAbsolutePath() + "': " + e.getMessage());
		} catch (ClassCastException e) {
			logger.logError("Invalid casting '" + model.eClass().getName() + "' to 'MProject' while loading model from file '" + xmiFile.getAbsolutePath() + "': " + e.getMessage());
		}
		return project;
	}

}
