
/****************************************************************************************
#  Copyright (c) 2011, ALES S.r.l. - Via C. Bartolomeo Piazza 13, 00161 Roma Italy      #
#  Developed with the sponsorship of the Defense Advanced Research Projects             #
#  Agency (DARPA).                                                                      #
#                                                                                       #
#  Delivered to the Government with Unlimited Rights under Contract #FA8650-10-C-7074.  #
#                                                                                       #
#  THE DATA IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR               #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,             #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE          #
#  AUTHORS, SPONSORS, DEVELOPERS, CONTRIBUTORS, OR COPYRIGHT HOLDERS BE LIABLE          #
#  FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF                   #
#  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE          #
#  DATA OR THE USE OR OTHER DEALINGS IN THE DATA.                                       #
****************************************************************************************/

package com.eu.ales.rpyplugins.utils;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Collections;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

public class Serialization {

	/**
	 * Load model from xmi file. It assumes that the model has a root, otherwise it returns the first element in the xmi resource
	 * @param xmiFile
	 * @return
	 * @throws IOException
	 */
	public static EObject loadRootFromFile (File xmiFile, String pkgURI, EPackage pkgInstance) throws IOException {
		Resource resource = createResource(xmiFile, pkgURI, pkgInstance);
		resource.load (Collections.EMPTY_MAP); 
		return resource.getContents().get(0);
	}

	public static Collection<EObject> loadAllFromFile (File xmiFile, String pkgURI, EPackage pkgInstance) throws IOException {
		Resource resource = createResource(xmiFile, pkgURI, pkgInstance);
		resource.load (Collections.EMPTY_MAP); 
		return resource.getContents();
	}

	/**
	 * Save model to xmi file.
	 * @param outputFile
	 * @param eObject
	 * @throws IOException
	 */
	public static void saveToFile(File xmiFile, EObject eObject, String pkgURI, EPackage pkgInstance) throws IOException {
		Resource resource = addEObjectToResource(eObject, xmiFile, pkgURI, pkgInstance);
		resource.save(Collections.EMPTY_MAP); 
	}

	/**
	 * Create resource for file.
	 * @param file
	 * @return
	 */
	public static Resource createResource(File file, String pkgURI, EPackage pkgInstance) {
		// Create a resource set to hold the resources.
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register the appropriate resource factory to handle all file extensions.
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
		(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());

		// Register the package to ensure it is available during writing.
		resourceSet.getPackageRegistry().put(pkgURI, pkgInstance);
		// create file resource URI 
		URI modelURI = URI.createFileURI(file.getAbsolutePath());
		return resourceSet.createResource(modelURI);
	}

	public static Resource addEObjectToResource(EObject eObject, File file, String pkgURI, EPackage pkgInstance) {
		if (eObject == null) {
			throw new Error("Invalid null object argument");
		}
		// go to root object
		while (eObject.eContainer() != null) {
			eObject = eObject.eContainer();
		}
		Resource resource = createResource(file, pkgURI, pkgInstance);
		resource.getContents().add(eObject);
		return resource;
	}

}
